@@ -1,4 +1,3 @@
-
 // easp.cpp
 //
 // Control panel applet for easpanel
@@ -24,6 +23,7 @@
 
 #include <QApplication>
 #include <QDir>
+#include <QKeyEvent>
 #include <QMessageBox>
 #include <QStringList>
 
@@ -36,11 +36,12 @@
 #include "../../icons/easpanel-22x22.xpm"
 
 MainWidget::MainWidget(QWidget *parent)
-  : QWidget(parent)
+  : QWidget(parent,Qt::CustomizeWindowHint|Qt::WindowMinimizeButtonHint|Qt::WindowMaximizeButtonHint)
 {
   main_auto=false;
   main_selected_alert_id=-1;
   main_next_is_voicetrack=false;
+  setFocusPolicy(Qt::StrongFocus);
 
   new CmdSwitch("easp","\n");
 
@@ -96,7 +97,8 @@ MainWidget::MainWidget(QWidget *parent)
   //
   // Text Editor
   //
-  main_text_text=new QTextEdit(this);
+  main_text_text=new AlertTextEdit(this);
+  connect(main_text_text,SIGNAL(quitRequested()),this,SLOT(quit()));
 
   //
   // Alert Buttons
@@ -112,11 +114,11 @@ MainWidget::MainWidget(QWidget *parent)
   //
   // Buttons
   //
-  main_auto_button=
-    new QPushButton(tr("LiveAssist"),this);
+  main_auto_button=new ModeButton(tr("LiveAssist"),this);
   main_auto_button->setFont(mode_font);
   main_auto_button->setStyleSheet("background-color: #FFFF00");
   connect(main_auto_button,SIGNAL(clicked()),this,SLOT(autoData()));
+  connect(main_auto_button,SIGNAL(quitRequested()),this,SLOT(quit()));
 
   main_livesend_button=
     new QPushButton(tr("Send to Log")+"\n("+tr("LIVE")+")",this);
@@ -189,18 +191,34 @@ void MainWidget::liveSendData()
     //
     // Load from the bottom up
     //
+    SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Outro Cart
+			      1,
+			      main_config->rivendellLiveassistFriendlyOutroCart(),offset));
+
     SendRml(QString().sprintf("PX %d %d %d STOP!",    // EOM
-			      main_config->rivendellLogMachine(),
+			      1,
 			      alert->eomCart(),offset));
     button->setLastCart(alert->eomCart());
 
-    SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Attention Signal
-			      main_config->rivendellLogMachine(),
-			      main_config->rivendellAlertToneCart(),offset));
+    if(alert->attentionCart()!=0) {
+      SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Silence
+				1,
+				alert->silenceCart(),offset));
+      SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Attention Signal
+				1,
+				alert->attentionCart(),offset));
+      SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Silence
+				1,
+				alert->silenceCart(),offset));
+    }
 
     SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Header
-			      main_config->rivendellLogMachine(),
+			      1,
 			      alert->headerCart(),offset));
+
+    SendRml(QString().sprintf("PX %d %d %d STOP!",    // Intro
+			      1,
+			      main_config->rivendellLiveassistFriendlyIntroCart(),offset));
     main_alert_buttons[main_selected_alert_id]->setStatus(AlertButton::Sent);
   }
 }
@@ -220,22 +238,30 @@ void MainWidget::cannedSendData()
     // Load from the bottom up
     //
     SendRml(QString().sprintf("PX %d %d %d PLAY!",    // EOM
-			      main_config->rivendellLogMachine(),
+			      1,
 			      alert->eomCart(),offset));
     button->setLastCart(alert->eomCart());
 
     if(alert->messageCart()!=0) {
       SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Message
-				main_config->rivendellLogMachine(),
+				1,
 				alert->messageCart(),offset));
+    }
 
+    if(alert->attentionCart()!=0) {
+      SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Silence
+				1,
+				alert->silenceCart(),offset));
       SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Attention Signal
-				main_config->rivendellLogMachine(),
-				main_config->rivendellAlertToneCart(),offset));
+				1,
+				alert->attentionCart(),offset));
+      SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Silence
+				1,
+				alert->silenceCart(),offset));
     }
 
     SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Header
-			      main_config->rivendellLogMachine(),
+			      1,
 			      alert->headerCart(),offset));
     main_alert_buttons[main_selected_alert_id]->setStatus(AlertButton::Sent);
   }
@@ -255,41 +281,55 @@ void MainWidget::autoSendData(int id)
     //
     // Load from the bottom up
     //
-    if(main_config->rivendellFriendlyOutroCart()!=0) {
+    if(main_config->outroCart(alert->easType())!=0) {
       SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Outro Cart
-				main_config->rivendellLogMachine(),
-				main_config->rivendellFriendlyOutroCart(),
+				1,
+				main_config->outroCart(alert->easType()),
 				offset));
-      button->setLastCart(main_config->rivendellFriendlyOutroCart());
+      button->setLastCart(main_config->outroCart(alert->easType()));
     }
     else {
       button->setLastCart(alert->eomCart());
     }
 
     SendRml(QString().sprintf("PX %d %d %d PLAY!",    // EOM
-			      main_config->rivendellLogMachine(),
+			      1,
 			      alert->eomCart(),offset));
 
     if(alert->messageCart()!=0) {
       SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Message
-				main_config->rivendellLogMachine(),
+				1,
 				alert->messageCart(),offset));
+    }
 
-      SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Attention Signal
-				main_config->rivendellLogMachine(),
-				main_config->rivendellAlertToneCart(),offset));
+    if(alert->attentionCart()!=0) {
+      SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Silence
+				1,
+				alert->silenceCart(),offset));
+      SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Message
+				1,
+				alert->attentionCart(),offset));
+      SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Silence
+				1,
+				alert->silenceCart(),offset));
     }
 
     SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Header
-			      main_config->rivendellLogMachine(),
+			      1,
 			      alert->headerCart(),offset));
 
     if(main_config->rivendellFriendlyIntroCart()!=0) {
       SendRml(QString().sprintf("PX %d %d %d PLAY!",    // Intro Cart
-				main_config->rivendellLogMachine(),
-				main_config->rivendellFriendlyIntroCart(),
+				1,
+				main_config->introCart(alert->easType()),
 				offset));
     }
+    if(offset==0) {    // Start Immediately
+      SendRml("PB 1!");
+    }
+    else {
+      SendRml("PB 3!");
+    }
     main_alert_buttons[id]->setStatus(AlertButton::Sent);
   }
 }
@@ -308,9 +348,15 @@ void MainWidget::alertScanData()
   for(int i=0;i<files.size();i++) {
     if(!main_alerts.contains(files.at(i))) {
       Alert *alert=new Alert();
-      if(alert->load(main_config->pathsEasDataDirectory()+"/"+files.at(i))) {
-	main_alerts[files.at(i)]=alert;
-	if(ProcessNewAlert(alert)) {
+      main_alerts[files.at(i)]=alert;
+    }
+  }
+  for(QMap<QString,Alert *>::const_iterator it=main_alerts.begin();
+      it!=main_alerts.end();it++) {
+    it.value()->touch();
+    if(it.value()->touches()==4) {
+      if(it.value()->load(main_config->pathsEasDataDirectory()+"/"+it.key())) {
+	if(ProcessNewAlert(it.value())) {
 	  if(main_auto) {
 	    SendNextAlert();
 	  }
@@ -319,7 +365,7 @@ void MainWidget::alertScanData()
     }
   }
 
-  main_alert_scan_timer->start(5000);
+  main_alert_scan_timer->start(1000);
 }
 
 
@@ -360,6 +406,10 @@ void MainWidget::alertClosedData(int id)
     if(alert->messageCart()!=0) {
       main_config->removeCart(alert->messageCart(),&err_msg);
     }
+    if(alert->attentionCart()!=0) {
+      main_config->removeCart(alert->attentionCart(),&err_msg);
+      main_config->removeCart(alert->silenceCart(),&err_msg);
+    }
     QString filename=alert->filename();
     unlink((main_config->pathsEasDataDirectory()+"/"+filename).toUtf8());
     if(!alert->headerAudio().isEmpty()) {
@@ -408,6 +458,38 @@ void MainWidget::rlmReadyReadData()
 }
 
 
+void MainWidget::quit()
+{
+  QString err_msg;
+  Alert *alert=NULL;
+
+  //
+  // Clean up Rivendell carts
+  //
+  for(int i=0;i<EASP_ALERT_QUAN;i++) {
+    if((alert=main_alert_buttons[i]->alert())!=NULL) {
+      if(alert->headerCart()!=0) {
+	main_config->removeCart(alert->headerCart(),&err_msg);
+      }
+      if(alert->eomCart()!=0) {
+	main_config->removeCart(alert->eomCart(),&err_msg);
+      }
+      if(alert->messageCart()!=0) {
+	main_config->removeCart(alert->messageCart(),&err_msg);
+      }
+      if(alert->attentionCart()!=0) {
+	main_config->removeCart(alert->attentionCart(),&err_msg);
+      }
+      if(alert->silenceCart()!=0) {
+	main_config->removeCart(alert->silenceCart(),&err_msg);
+      }
+    }
+  }
+
+  exit(0);
+}
+
+
 void MainWidget::resizeEvent(QResizeEvent *e)
 {
   int w=size().width();
@@ -432,29 +514,18 @@ void MainWidget::resizeEvent(QResizeEvent *e)
 }
 
 
+void MainWidget::keyPressEvent(QKeyEvent *e)
+{
+  if((e->key()==Qt::Key_X)&&((e->modifiers()&Qt::AltModifier)!=0)) {
+    quit();
+  }
+  QWidget::keyPressEvent(e);
+}
+
+
 void MainWidget::closeEvent(QCloseEvent *e)
 {
-  QString err_msg;
-  Alert *alert=NULL;
-
-  //
-  // Clean up Rivendell carts
-  //
-  for(int i=0;i<EASP_ALERT_QUAN;i++) {
-    if((alert=main_alert_buttons[i]->alert())!=NULL) {
-      if(alert->headerCart()!=0) {
-	main_config->removeCart(alert->headerCart(),&err_msg);
-      }
-      if(alert->eomCart()!=0) {
-	main_config->removeCart(alert->eomCart(),&err_msg);
-      }
-      if(alert->messageCart()!=0) {
-	main_config->removeCart(alert->messageCart(),&err_msg);
-      }
-    }
-  }
-
-  exit(0);
+  quit();
 }
 
 
@@ -525,6 +596,26 @@ bool MainWidget::ProcessNewAlert(Alert *alert)
       else {
 	alert->setEomCart(cartnum);
       }
+      if(!alert->attentionAudio().isEmpty()) {
+	if((cartnum=main_config->
+	    importCart("*** EAS ATTENTION SIGNAL *** ["+alert->title()+"]",
+		       alert->attentionAudio(),&err_msg))==0) {
+	  button->setStatus(AlertButton::Error);
+	  button->addStatusText("Missing attention signal audio ["+err_msg+"]. ");
+	}
+	else {
+	  alert->setAttentionCart(cartnum);
+	}
+	if((cartnum=main_config->
+	    importCart("*** EAS SILENCE *** ["+alert->title()+"]",
+		       "/usr/share/easpanel/silence.wav",&err_msg))==0) {
+	  button->setStatus(AlertButton::Error);
+	  button->addStatusText("Missing silence audio ["+err_msg+"]. ");
+	}
+	else {
+	  alert->setSilenceCart(cartnum);
+	}
+      }
       if(!alert->messageAudio().isEmpty()) {
 	if((cartnum=main_config->
 	    importCart("*** EAS MESSAGE *** ["+alert->title()+"]",
